/*-
 * Copyright (C) 2012, Noriaki TANAKA <dtana@startide.jp>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY AUTHOR AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL AUTHOR OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: rnikon.c,v 1.7 2012/01/25 17:15:05 dtana Exp $
 *
 * rnikon - nikon 10pin controller through USB-IO 2.0(aki)
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <usb.h>

static struct usb_device *rnikon_find(int, int);
static struct usb_dev_handle *rnikon_open(struct usb_device *);
static int rnikon_close(struct usb_dev_handle *);
static int rnikon_send(struct usb_dev_handle *, unsigned char *);

#define USB_VENDOR 0x1352
#define USB_IO_AKI 0x0121

static void
usb_perror(const char *s)
{
	fprintf(stderr, "%s: %s\n", s, usb_strerror());
}

#define DATASIZE 64

int
main(int argc, char **argv)
{
	struct usb_device *dp;
	usb_dev_handle *uh;
	unsigned char data[DATASIZE];
	int on = 0;
	
	if (argc == 2 && strcmp(argv[1], "on") == 0)
		on = 0x04;
	
	if ((dp = rnikon_find(USB_VENDOR, USB_IO_AKI)) == NULL)
		usb_perror("find"), exit(1);
	if ((uh = rnikon_open(dp)) == NULL)
		usb_perror("open"), exit(1);
	
	memset(data, 0, sizeof(data));
	data[0] = 0x20; /* command: digital I/O */
	data[1] = 0x01; /* port J1 */
	data[2] = on;   /* J1-0=0x01, J1-1=0x02, J1-2=0x04 ... */
	if (rnikon_send(uh, data) < 0)
		usb_perror("send"), exit(1);
	
	/* needs? */
	memset(data, 0, sizeof(data));
	data[0] = 0x2A; /* command: analog input */
	data[1] = 0x01; /* channel 1 */
	data[4] = 0x02; /* channel 2 */
	if (rnikon_send(uh, data) < 0)
		usb_perror("send"), exit(1);

	if (rnikon_close(uh) < 0)
		usb_perror("close"), exit(1);
	
	return 0;
}

static struct usb_device *
rnikon_find(int vid, int pid)
{
	struct usb_bus *bus;
	struct usb_device *dp;
	
	usb_init();
	if (usb_find_busses() < 0 || usb_find_devices() < 0)
		return NULL;
	if ((bus = usb_get_busses()) == NULL)
		return NULL;
	
	for (; bus; bus = bus->next)
		for(dp = bus->devices; dp; dp = dp->next)
			if (dp->descriptor.idVendor  == vid &&
			    dp->descriptor.idProduct == pid)
				return dp;
	return NULL;
}

static int
detach_kernel_driver(struct usb_dev_handle *uh, struct usb_device *dp)
{
#ifdef __FreeBSD__
	return -1;
#else
	return usb_detach_kernel_driver_np(uh,
	    dp->config->interface->altsetting->bInterfaceNumber);
#endif
}

static struct usb_dev_handle *
rnikon_open(struct usb_device *dp)
{
	struct usb_dev_handle *uh;
	
	if ((uh = usb_open(dp)) == NULL)
		return NULL;
	if (usb_set_configuration(uh, dp->config->bConfigurationValue) < 0)
		if (detach_kernel_driver(uh, dp) < 0)
			return NULL;
	if (usb_claim_interface(uh,
		dp->config->interface->altsetting->bInterfaceNumber) < 0)
		return NULL; 
	return uh;
}

static int
rnikon_close(struct usb_dev_handle *uh)
{
	return usb_close(uh);
}

static int
rnikon_send(struct usb_dev_handle *uh, unsigned char *data)
{
	static unsigned char seq = 0;
	
	data[DATASIZE - 1] = seq++;
	if (usb_bulk_write(uh, 1, (char *)data, DATASIZE, 1000) != DATASIZE)
		return -1;
	return 0;
}
